function [out]=apwl1(y,X,L,options)
% Adaptive projection-based algorithm using weighted $\ell_1$ balls
% For details see [1].
% This function, at this stage, is not optimized with respect to speed
% or with respect to memory consumption. 
%
% REFERENCES
%
%[1] Y. Kopsinis, K. Slavakis, S. Theodoridis, "Online Sparse System Identification and Signal Reconstruction Using
%Projections Onto Weighted $ell_{1}$ Balls," Signal Processing, IEEE Transactions on , vol.59, no.3, pp.936-952, March 2011
%doi: 10.1109/TSP.2010.2090874
%
% Ver. 1.0
% Yannis Kopsinis, 10/02/2011
% kopsinis@ieee.org




q=options.q; % Number of concurrently processed Hyperslabs

% Set up of the extrapolation parameter
mu=options.mu;
if length(mu)==2
    UseExtrapolation=mu(1);
else
    UseExtrapolation=1;
end

if ( UseExtrapolation == 0 )
    muNoExtrapolation=mu(2);
else
    muOverMu=mu;
end
%______________________________________


delta=options.Delta;

epsilon=options.SlabWidth;

if length(options.WeightEps)==2
   epsilonprimetype=1;
   epsilonprime0 = options.WeightEps(1);
   epsilonprime = options.WeightEps(2); 
else
   epsilonprimetype=2;
   epsilonprime = options.WeightEps; 
end
verbose=options.Verbose;

N=length(y); % Number of observations

h=zeros(L,1); 

if length(options.Output)>1
   outpos=options.Output;
   outpos(outpos>N)=0;
   Noutpos=length(outpos);
   outindex=1;
   options.Output=3;
   out=zeros(L,Noutpos);
elseif options.Output==2
    out=zeros(L,N);
elseif options.Output==1
    out=zeros(L,1);
end
outtype=options.Output;


A=zeros(L,q);
x2v=zeros(1,q);

epsilondiv=0;

proj=zeros(L,q); betaf=zeros(1,q); betafindex=zeros(1,q);


%% Start counting iterations %%%%%%%%%
if verbose, fprintf('  --- Start iterations --- \n'), end

for i=1:N
    
    
    
    J=i:-1:max(1,i-q+1);

    yn=y(J);

    %x2v=X2(J);

    qq=length(J);
  
    nzr = find (h~=0);

    % computes $ \bm{h}^T \bm{x}_n - y_n $ 
    if size(X,2)==1;
        xn=X(i+L-1:-1:i);
        A=circshift(A,[0,1]);
        A(:,1)=xn;
       % XX=A(nzr,1:qq)'*h(nzr)-yn;
   
        x2v=circshift(x2v,[0,1]);
        x2v(1)=xn'*xn;
    else
        A=circshift(A,[0,1]);
        xn=X(i,:)';
        A(:,1)=xn;
        
      %  XX=X(nzr,J)'*h(nzr)-yn; 

        x2v=circshift(x2v,[0,1]); 
        x2v(1)=xn'*xn;
      
    end
    XX=A(nzr,1:qq)'*h(nzr)-yn;
    %____________________________________
    
    NoActive=0;

    % computes the right term of the addition in eq.(12)
    for ih=1:qq
        if XX(ih)<-epsilon
            NoActive=NoActive+1;
            betaf(NoActive)=(-XX(ih)-epsilon)/x2v(ih); 
            betafindex(NoActive)=ih;
            % proj(:,NoActive)=-betaf(NoActive)*X(:,J(ih));
            proj(:,NoActive)=-betaf(NoActive)*A(:,ih);

        elseif XX(ih)>epsilon
            NoActive=NoActive+1;
            betaf(NoActive)=(-XX(ih)+epsilon)/x2v(ih); 
            betafindex(NoActive)=ih;
            % proj(:,NoActive)=-betaf(NoActive)*X(:,J(ih));
            proj(:,NoActive)=-betaf(NoActive)*A(:,ih);
        end
    end
    %___________________________________________________
    
    
    if ( NoActive == 0 )
        Mu=1;
        Step=zeros(L,1);
    else
        % There are activy hyperslabs
        AllProj=proj(:,1:NoActive);
        
        % All \omega_j are set equal to 1/NoActive, where NoActive is the
        % size of the set \mathcal{I}_n
        
        Omega=ones(NoActive,1)/NoActive; 
       %______________________________________________________________
        
        
        % Computes the quantity in the internal brackets of the main equation Eq. (9)
        % (after proper reformulation)
        Step=AllProj*Omega;
       %________________________________________________________________
        
   
        if ( UseExtrapolation == 1 )
            % Compute the numerator of eq.(10) (after proper reformulation of
            % \mathcal{M}_n)
   
            NumMu=1/NoActive*(sum( (betaf(1:NoActive).^2) .* x2v(betafindex(1:NoActive)) ));
           %______________________________________________________________     
        
            if any(Step)
                Mu=NumMu/(norm(Step)^2);
            else
                Mu=1;
            end
        end   
    end
    
    if ( UseExtrapolation == 0 )
        mu=muNoExtrapolation;
    else
        mu=Mu*muOverMu;
    end
    
    % Adaptation step. Computes the quantity inside the outer-brackets of eq. (9)
    h=h-mu*Step;
    %______________________________________________________________________
    
    % Computation of $\epsilon^'_n$ in eq. (13) either adaptively or it is set to a fixed value.
    if epsilonprimetype==1
            epsilondiv=epsilondiv+1;
            epsilonprime_val=epsilonprime0+epsilonprime/epsilondiv;
    elseif epsilonprimetype==2
        epsilonprime_val=epsilonprime;
    end
    %________________________________________________________________
    
    
    % Computation of the weights
    w=1./(abs(h)+epsilonprime_val);
    %_______________________________________________________________
    
    
    % Projection onto the weighted l1 ball
    h=l1weightedprj(h,w,delta);
    %_______________________________________________________________
    
    
    if outtype==2
       out(:,i)=h;
    elseif outtype==3
       if outindex<=Noutpos
           if i==outpos(outindex);
               out(:,outindex)=h;
               outindex=outindex+1;
           end
       end
    end
    
    
    if verbose, disprog(i,N,10), end


end

if outtype==1
    out=h;
end

end