function options = spaloptions(varargin)

%   Function SPALOPTIONS constructs a structure array with fields the parameters needed in the
%   SPAL function.
%
%   options = spaloptions('Option_name',Option_value)
%   or
%   options = spaloptions(@METHOD,'Option_name',Option_value)
%
%   INPUT (optional)
%         @METHOD Either:
%             1) @APWL1 (For projections onto the weighted L1 ball)
%             2) @APL1  (For projections onto the L1 ball)
%
%         'Option_name'- Option_value pairs as they are described below (help spaloptions)
%     -------------------------------------------------------------------------------
%
%   OUTPUT
%         options:  A structure array (options structure) with fields the parameters needed in the SPAL function.
%     -------------------------------------------------------------------------------
%
%   USAGE
%
%         SPALOPTIONS (with no input arguments) displays a list of all the available parameters and
%         their valid values.
%
%         OPTIONS = SPALOPTIONS(@METHOD) creates an options structure with the default parameters used in the specific METHOD. If @METHOD is missing, then @APWL1 is assumed.
%
%         OPTIONS = SPALOPTIONS(@METHOD,'option_name1',OPTION_VALUE1,'option_name2',OPTION_VALUE2,...) creates an options structure with
%         parameters OPTION_VALUE1, OPTION_VALUE2, etc. for the specified options 'option_name1', 'option_name2' etc and sets
%         the rest of the parameters associated with @METHOD to their default values.
%         If @METHOD is not specified the APWL1 method is assumed.
%
%         OPTIONS = SPALOPTIONS(OLDOPTS,'option_name1',OPTION_VALUE1,'option_name2',OPTION_VALUE2,...) will
%         reassign those fields in OLDOPTS specified by 'option_name1', 'option_name2',
%         etc, to OPTION_VALUE1, OPTION_VALUE2, etc
%     ---------------------------------------------------------------------
%
%
%
%  Several OPTIONS and their NAMES
%
%   Method: - Method applied. It can be @APWL1 or @APL1. It is equivalent
%             to use @METHOD as first argument (see above)
%
%   q:    - Positive integer indicating the number of successive hyperslabs used.
%
%   mu:   - Specifies the extrapolation parameter $\mu_n$. It can be either
%             1)  A real number (0 1]. In this case $\mu_n = mu * M_n$
%             2)  A two component vector where the first one equals to 0.
%                 In this case no extrapolation is used and $\mu_n = mu$.
%
%   SlabWidth:   - Positive real which specifies the Hyperslab width $\epsilon$.
%
%   Delta:  - radius of the \ell_1 ball. In case of @APL1 should be set equal or larger than norm(h,1)
%             and in case of @APWL1 should be set equal or larger to length(find(h==0)), where h is
%             the sparse unknown vector
%
%   WeightEps:  - It determines parameter $\epsilon^'_n$ (see [1, eq. 13]) in order to avoid
%                 division by zero. There are two options
%                 1) weighteps is a scalar, then $\epsilon^'_n$=weighteps \forall n.
%                 2) weighteps is a vector with two components, then $\epsilon^'_n$=weighteps(1)+weighteps(2)/(n+1)
%
%   Output: - It determines the output of the algorithm. There are three options:
%                 1) To be a scalar with value 1. In this case a vector of length L of the finally estimated unknown vector, $\bm{h}_N$, is returned.
%                 2) To be a scalar with value 2. A matrix H of size LxN, where L is the length of h and N the number of observations, is returned. Each column of H, say the i-th one, contains the 
%                     tentative estimate of the unknown vector $\bm{h}_i$.
%                 3) To be a vector, say v, of length K, where K<=N, of positive integers, smaller or equal to N, in ascending order. The output is a matrix H LxK, 
%                     where each column of H, say the i-th one, contains the tentative estimate of the unknown vector after processing v(i) estimates, i.e, it contains $\bm{h}_j$, where j=v(i).
% 
%   Verbose:  - Set equal to 1 to allow algorithm progress to be displayed. Set zero for "silent mode".
%
%   ---------------------------------------------------------------------
%
% The Parameters are not case sensitive.
%
%   EXAMPLES:
%       options = spaloptions(@apwl1,'q',20,'mu',1.5)
%       options = spaloptions(options,'delta', 5)
%       spaloptions
%
% Moreover, parameter names do not need to be fully entered.
%   For Example
%       options = spalparam(@APL1,'SlabWidth', 0.03)
%       is equivalent to
%       options = spalparam(@APL1,'sla', 0.03)
%
%
% REFERENCES
%
%[1] Y. Kopsinis, K. Slavakis, S. Theodoridis, "Online Sparse System Identification and Signal Reconstruction Using
%Projections Onto Weighted $ell_{1}$ Balls," Signal Processing, IEEE Transactions on , vol.59, no.3, pp.936-952, March 2011
%doi: 10.1109/TSP.2010.2090874
%
% Ver. 1.0
% Yannis Kopsinis, 10/02/2011
% kopsinis@ieee.org

%%% Initializing for the case the SPALOPTIONS.m is called inside SPAL.m
if nargin==1
    if iscell(varargin{1})
        varargin=varargin{1};
        nargin2=length(varargin);
    else
        nargin2=nargin;
    end
else
    nargin2=nargin;
end

if nargin2>0
    if isa(varargin{1},'function_handle')
        funcname = lower(func2str(varargin{1}));
    else
        funcname = 'apwl1';
    end
else
    funcname='apwl1';
end
optionsstandart=makeoptionsstandart(funcname);

Names = fieldnames(optionsstandart);

names = lower(Names);


if nargin2==0 && nargout==1
    options=optionsstandart;
    %     fprintf(' q has not been specified. Please specify. \n \n')
    %     fprintf('Hypewidth has not been specified. Please specify. A value about 1.3 times the noise standard deviation, is a good option \n \n')
    %     fprintf('Delta has not been specified. Please specify.  \n')
    return
    
elseif nargin2==0 && nargout==0
    fprintf('       Method: @APWL1 or @APL1 \n')
    fprintf('       q: positive scalar \n')
    fprintf('       mu: Real in (0,1]  \n')
    fprintf('       SlabWidth: positive scalar  (real) \n')
    fprintf('       Delta: possitive scalar (real) \n')
    fprintf('       WeightEps: Either a scalar or a two component vector \n')
    fprintf('       Output: Either a scalar or a vector \n')
    fprintf('       Verbose: 1 or 0 \n')
    
    return
    
    
elseif isa(varargin{1},'function_handle')
    funcname = lower(func2str(varargin{1}));
    if strcmp(funcname,'apwl1')==1 || strcmp(funcname,'apl1')==1
        if length(varargin)>1
            if isa(varargin{2},'struct')
                optionsdefaults=varargin{2};
                %optionsdefaults=setfield(optionsdefaults,'Method',upper(varargin{1}));
                optionsdefaults.Method=upper(varargin{1});
                varargin(1)=[];
            else
                optionsdefaults=optionsstandart;
                %optionsdefaults=setfield(optionsdefaults,'Method',upper(varargin{1}));
                optionsdefaults.Method=upper(varargin{1});
            end
        else
            optionsdefaults=optionsstandart;
            %optionsdefaults=setfield(optionsdefaults,'Method',upper(varargin{1}));
            optionsdefaults.Method=upper(varargin{1});
        end
    else
        error(['Method ',funcname,' is not known'])
    end
    varargin(1)=[];
elseif isa(varargin{1},'struct')
    optionsdefaults=varargin{1};
    varargin(1)=[];
else
    optionsdefaults=optionsstandart;
end


numbofargs=length(varargin);
if numbofargs==0
    options=optionsdefaults;
elseif mod(numbofargs,2)==1
    error('Arguments must occur in name-value pairs.')
else
    options=optionsdefaults;
    for arg=1:2:numbofargs
        in_arg=varargin{arg};
        in_val=varargin{arg+1};
        if ~ischar(in_arg)
            error('Expected argument %d to be a string parameter name.', arg)
        end
        findattempt=strmatch(lower(in_arg),names);
        if isempty(findattempt)
            error('There is not a parameter called ''%s''.',in_arg)
        elseif length(findattempt) > 1
            findattempt=strmatch(lower(in_arg),names,'exact');
        end
        
        if ischar(in_val)
            in_val = (deblank(in_val));
        end
        check_arg(Names{findattempt},in_val);
        options.(Names{findattempt,:}) = in_val;
    end
end

end

function check_arg(arg,val)
switch arg
    case {'Method'}
        if ~isa(val,'function_handle')
            error('Invalid value, option ''%s'' should be a function_handle.',arg);
        else
            funcname = lower(func2str(val));
            if strcmp(funcname,'apwl1')~=1 && strcmp(funcname,'apl1')~=1
                error('The method ''%s'' is not known',funcname)
            end
        end
    case {'q','Verbose'}
        if ~isnumeric(val)
            error('Invalid value for parameter ''%s''.',arg);
        else
            if val~=round(val)
                error('parmeter ''%s'' must be integer number.',arg);
            end
        end
    case {'Output'}
        if ~isnumeric(val)
            error('Invalid value for parameter ''%s''.',arg);
        else
            erval=0;
            for i=1:length(val)
                if val(i) ~=round(val(i))
                    erval=1;
                end
            end
            if erval==1
                error(' ''%s'' must contain integer number(s).',arg);
            end
            if length(val)>1
                if ~all(diff(val)>0)
                    error(' ''%s'' must have values in ascending order all different to each other.',arg);
                end
            end
        end
    case {'mu','SlabWidth','Delta','WeightEps'}
        if ~isnumeric(val)
            error('Invalid value. Parameter %s takes numeric inputs.',arg)
        end
        
    otherwise
        error('Unknown parameter name.')
end

end

function opt= makeoptionsstandart(funcname)

opt=struct('Method',[],...
    'q',[],...
    'mu',[],...
    'SlabWidth',[],...
    'Delta',[],...
    'WeightEps',[],...
    'Output',1,...
    'Verbose',1);

if strcmp(funcname,'apwl1')
    opt.Method=@APWL1;
    opt.mu=1.8;
    opt.WeightEps=0.05;
elseif strcmp(funcname,'apl1')
    opt.Method=@APL1;
    opt.mu=1.8;
end


end
